@@ -7,12 +7,16 @@ use p256::{
         signature::{Signer, Verifier},
         Signature, SigningKey, VerifyingKey,
     },
-    elliptic_curve::{ecdh::diffie_hellman, sec1::Coordinates, Curve, SecretValue},
+    elliptic_curve::{
+        self,
+        ecdh::diffie_hellman,
+        sec1::{Coordinates, FromEncodedPoint, ToEncodedPoint},
+    },
     EncodedPoint, PublicKey, SecretKey,
 };
 use subtle::ConstantTimeEq;
 
-use super::{EcCurves, HasKeyAlg, KeyAlg};
+use super::{ec_common, EcCurves, HasKeyAlg, KeyAlg};
 use crate::{
     buffer::{ArrayKey, WriteBuffer},
     error::Error,
@@ -47,7 +51,7 @@ pub static JWK_KEY_TYPE: &'static str = "EC";
 /// The 'crv' value of a P-256 key JWK
 pub static JWK_CURVE: &'static str = "P-256";
 
-type FieldSize = <p256::NistP256 as Curve>::FieldSize;
+type FieldSize = elliptic_curve::FieldSize<p256::NistP256>;
 
 /// A P-256 (secp256r1) public key or keypair
 #[derive(Clone, Debug)]
@@ -115,10 +119,8 @@ impl KeyGen for P256KeyPair {
     fn generate(mut rng: impl KeyMaterial) -> Result<Self, Error> {
         ArrayKey::<FieldSize>::temp(|buf| loop {
             rng.read_okm(buf);
-            if let Some(key) = p256::NistP256::from_secret_bytes(&buf) {
-                if key.is_zero().unwrap_u8() == 0 {
-                    return Ok(Self::from_secret_key(SecretKey::new(key)));
-                }
+            if let Ok(key) = SecretKey::from_be_bytes(&buf) {
+                return Ok(Self::from_secret_key(key));
             }
         })
     }
@@ -127,14 +129,16 @@ impl KeyGen for P256KeyPair {
 impl KeySecretBytes for P256KeyPair {
     fn from_secret_bytes(key: &[u8]) -> Result<Self, Error> {
         Ok(Self::from_secret_key(
-            SecretKey::from_bytes(key).map_err(|_| err_msg!(InvalidKeyData))?,
+            SecretKey::from_be_bytes(key).map_err(|_| err_msg!(InvalidKeyData))?,
         ))
     }
 
     fn with_secret_bytes<O>(&self, f: impl FnOnce(Option<&[u8]>) -> O) -> O {
         if let Some(sk) = self.secret.as_ref() {
-            let b = p256::SecretBytes::from(sk.to_bytes());
-            f(Some(&b[..]))
+            ArrayKey::<FieldSize>::temp(|arr| {
+                ec_common::write_sk(sk, &mut arr[..]);
+                f(Some(&arr))
+            })
         } else {
             f(None)
         }
@@ -158,12 +162,11 @@ impl KeypairBytes for P256KeyPair {
     }
 
     fn with_keypair_bytes<O>(&self, f: impl FnOnce(Option<&[u8]>) -> O) -> O {
-        if let Some(secret) = self.secret.as_ref() {
+        if let Some(sk) = self.secret.as_ref() {
             ArrayKey::<<Self as KeypairMeta>::KeypairSize>::temp(|arr| {
-                let sk_b = p256::SecretBytes::from(secret.to_bytes());
-                let pk_enc = EncodedPoint::encode(self.public, true);
-                arr[..SECRET_KEY_LENGTH].copy_from_slice(&sk_b[..]);
-                arr[SECRET_KEY_LENGTH..].copy_from_slice(pk_enc.as_ref());
+                ec_common::write_sk(sk, &mut arr[..SECRET_KEY_LENGTH]);
+                let pk_enc = self.public.to_encoded_point(true);
+                arr[SECRET_KEY_LENGTH..].copy_from_slice(pk_enc.as_bytes());
                 f(Some(&*arr))
             })
         } else {
@@ -174,9 +177,7 @@ impl KeypairBytes for P256KeyPair {
 
 impl KeyPublicBytes for P256KeyPair {
     fn from_public_bytes(key: &[u8]) -> Result<Self, Error> {
-        let pk = EncodedPoint::from_bytes(key)
-            .and_then(|pt| pt.decode())
-            .map_err(|_| err_msg!(InvalidKeyData))?;
+        let pk = PublicKey::from_sec1_bytes(key).map_err(|_| err_msg!(InvalidKeyData))?;
         Ok(Self {
             secret: None,
             public: pk,
@@ -184,8 +185,7 @@ impl KeyPublicBytes for P256KeyPair {
     }
 
     fn with_public_bytes<O>(&self, f: impl FnOnce(&[u8]) -> O) -> O {
-        let pt = EncodedPoint::encode(self.public, true);
-        f(pt.as_ref())
+        f(self.public.to_encoded_point(true).as_bytes())
     }
 }
 
@@ -197,7 +197,7 @@ impl KeySign for P256KeyPair {
         out: &mut dyn WriteBuffer,
     ) -> Result<(), Error> {
         match sig_type {
-            None | Some(SignatureType::ES256K) => {
+            None | Some(SignatureType::ES256) => {
                 if let Some(sig) = self.sign(message) {
                     out.buffer_write(&sig[..])?;
                     Ok(())
@@ -227,8 +227,8 @@ impl KeySigVerify for P256KeyPair {
 }
 
 impl ToJwk for P256KeyPair {
-    fn encode_jwk(&self, enc: &mut JwkEncoder<'_>) -> Result<(), Error> {
-        let pk_enc = EncodedPoint::encode(self.public, false);
+    fn encode_jwk(&self, enc: &mut dyn JwkEncoder) -> Result<(), Error> {
+        let pk_enc = self.public.to_encoded_point(false);
         let (x, y) = match pk_enc.coordinates() {
             Coordinates::Identity => {
                 return Err(err_msg!(
@@ -237,7 +237,7 @@ impl ToJwk for P256KeyPair {
                 ))
             }
             Coordinates::Uncompressed { x, y } => (x, y),
-            Coordinates::Compressed { .. } => unreachable!(),
+            Coordinates::Compressed { .. } | Coordinates::Compact { .. } => unreachable!(),
         };
 
         enc.add_str("crv", JWK_CURVE)?;
@@ -259,6 +259,12 @@ impl ToJwk for P256KeyPair {
 
 impl FromJwk for P256KeyPair {
     fn from_jwk_parts(jwk: JwkParts<'_>) -> Result<Self, Error> {
+        if jwk.kty != JWK_KEY_TYPE {
+            return Err(err_msg!(InvalidKeyData, "Unsupported key type"));
+        }
+        if jwk.crv != JWK_CURVE {
+            return Err(err_msg!(InvalidKeyData, "Unsupported key algorithm"));
+        }
         let pk_x = ArrayKey::<FieldSize>::try_new_with(|arr| {
             if jwk.x.decode_base64(arr)? != arr.len() {
                 Err(err_msg!(InvalidKeyData))
@@ -273,9 +279,10 @@ impl FromJwk for P256KeyPair {
                 Ok(())
             }
         })?;
-        let pk = EncodedPoint::from_affine_coordinates(pk_x.as_ref(), pk_y.as_ref(), false)
-            .decode()
-            .map_err(|_| err_msg!(InvalidKeyData))?;
+        let pk = Option::from(PublicKey::from_encoded_point(
+            &EncodedPoint::from_affine_coordinates(pk_x.as_ref(), pk_y.as_ref(), false),
+        ))
+        .ok_or_else(|| err_msg!(InvalidKeyData))?;
         if jwk.d.is_some() {
             ArrayKey::<FieldSize>::temp(|arr| {
                 if jwk.d.decode_base64(arr)? != arr.len() {
@@ -302,7 +309,7 @@ impl KeyExchange for P256KeyPair {
     fn write_key_exchange(&self, other: &Self, out: &mut dyn WriteBuffer) -> Result<(), Error> {
         match self.secret.as_ref() {
             Some(sk) => {
-                let xk = diffie_hellman(sk.secret_scalar(), other.public.as_affine());
+                let xk = diffie_hellman(sk.to_nonzero_scalar(), other.public.as_affine());
                 out.buffer_write(xk.as_bytes())?;
                 Ok(())
             }
@@ -335,7 +342,7 @@ mod tests {
 
         let jwk = sk.to_jwk_public(None).expect("Error converting key to JWK");
         let jwk = JwkParts::from_str(&jwk).expect("Error parsing JWK");
-        assert_eq!(jwk.kty, "EC");
+        assert_eq!(jwk.kty, JWK_KEY_TYPE);
         assert_eq!(jwk.crv, JWK_CURVE);
         assert_eq!(jwk.x, test_pub_b64.0);
         assert_eq!(jwk.y, test_pub_b64.1);
@@ -343,9 +350,9 @@ mod tests {
         let pk_load = P256KeyPair::from_jwk_parts(jwk).unwrap();
         assert_eq!(sk.to_public_bytes(), pk_load.to_public_bytes());
 
-        let jwk = sk.to_jwk_secret().expect("Error converting key to JWK");
+        let jwk = sk.to_jwk_secret(None).expect("Error converting key to JWK");
         let jwk = JwkParts::from_slice(&jwk).expect("Error parsing JWK");
-        assert_eq!(jwk.kty, "EC");
+        assert_eq!(jwk.kty, JWK_KEY_TYPE);
         assert_eq!(jwk.crv, JWK_CURVE);
         assert_eq!(jwk.x, test_pub_b64.0);
         assert_eq!(jwk.y, test_pub_b64.1);
